using System;
using UnityEngine;
using UnityEngine.Experimental.Rendering;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using SimpleJSON;
using System.Threading;
using MVR;
using MeshVR;
using System.Linq;


namespace Stopper {
	public class ClothingTransfer : MVRScript {
		DAZCharacterSelector characterSelector;
		
		string targetClothingUid;
		string targetAtomUid;
		
		
		public override void Init() {
			targetClothingUid = "Clothing Creator";
			targetAtomUid = containingAtom.uid;
			
			characterSelector = containingAtom.GetComponentInChildren<DAZCharacterSelector>();
			
			JSONStorableStringChooser clothingChooser = new JSONStorableStringChooser(
					"Target Clothing", 
					containingAtom.GetComponentsInChildren<DAZClothingItem>().Select(c => c.uid).ToList(),
					targetClothingUid, 
					"Target Clothing", 
					(string uid) => {
						this.targetClothingUid = uid;
					});
			CreateScrollablePopup(clothingChooser, false);
			
			
			JSONStorableStringChooser targetChooser = new JSONStorableStringChooser(
					"Target Atom", 
					SuperController.singleton.GetAtoms().Where(a => a.category == "People").Select(a => a.uid).ToList(), 
					targetAtomUid, 
					"Target Atom", 
					(string uid) => {
						this.targetAtomUid = uid;
					});
			CreateScrollablePopup(targetChooser, false);
			
			
			var goButton = CreateButton("Transfer!");
			var transferAction = new JSONStorableAction("Transfer", Transfer);
			transferAction.RegisterButton(goButton);
			RegisterAction(transferAction);
			
			JSONStorableFloat skinJointStrengthFloat = new JSONStorableFloat("Skin Joint Strength", 1f, 0f, 1f, true, true);
			RegisterFloat(skinJointStrengthFloat);
			var skinJointSlider = CreateSlider(skinJointStrengthFloat, false);
			skinJointSlider.valueFormat = "F3";
			
			skinJointStrengthFloat.setCallbackFunction = (float val) => {
				DAZClothingItem clothingItem = characterSelector.GetClothingItem(targetClothingUid);
				if(clothingItem == null){
					SuperController.LogMessage($"Could not find Clothing {targetClothingUid}");
					return;
				}
				
				ClothSimControl simControl = clothingItem.GetComponentInChildren<ClothSimControl>();
				if(simControl == null){
					return;
				}
				
				simControl.GetFloatJSONParam("jointStrength").val = val;
			};
		}
	
		private void Transfer(){
			try{
					
				DAZClothingItem clothingItem = characterSelector.GetClothingItem(targetClothingUid);
				if(clothingItem == null){
					SuperController.LogMessage($"Could not find Clothing {targetClothingUid}");
					return;
				}
				
				DAZSkinWrap skinWrap = clothingItem.gameObject.GetComponentInChildren<DAZSkinWrap>();
				
				Atom targetAtom = SuperController.singleton.GetAtomByUid(targetAtomUid);
				
				if(targetAtom == null){
					SuperController.LogMessage($"Could not find Atom {targetAtomUid}");
					return;
				}
				
				DAZMergedSkinV2 otherSkin = targetAtom.GetComponentInChildren<DAZMergedSkinV2>();
				if(otherSkin == null){
					SuperController.LogMessage($"Could not find skin on target atom. Is it a Person?");
					return;
				}
				
				skinWrap.skin = otherSkin;
			} catch (Exception e){
				SuperController.LogMessage($"Cloth transfer failed: {e}");
			}
		}
		
		private void Start(){
			
		}
	
			
		private void OnDisable()
		{
	
		}
		
		private void OnDestroy(){
		}
	}
}
